@@ -1,6 +1,6 @@
 /***************************************************************************
  *   sslscan - A SSL cipher scanning tool                                  *
- *   Copyright (C) 2007-2008 by Ian Ventura-Whiting (Fizz)                 *
+ *   Copyright 2007-2009 by Ian Ventura-Whiting (Fizz)                     *
  *   fizz@titania.co.uk                                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,6 +36,7 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
+#include <openssl/err.h>
 #include <openssl/ssl.h>
 #include <openssl/pkcs12.h>
 #include <openssl/x509.h>
@@ -76,11 +77,11 @@ const char *program_banner = "                   _\n"
                              "          / __/ __| / __|/ __/ _` | '_ \\\n"
                              "          \\__ \\__ \\ \\__ \\ (_| (_| | | | |\n"
                              "          |___/___/_|___/\\___\\__,_|_| |_|\n\n"
-                             "                    Version 1.7.1\n"
-                             "              http://www.titania.co.uk\n"
-                             "     Copyright (C) 2007-2008 Ian Ventura-Whiting\n";
-const char *program_version = "sslscan version 1.7.1\nhttp://www.titania.co.uk\nCopyright (C) 2007-2008 Ian Ventura-Whiting\n";
-const char *xml_version = "1.7";
+                             "                  Version 1.8.2\n"
+                             "             http://www.titania.co.uk\n"
+                             "        Copyright Ian Ventura-Whiting 2009\n";
+const char *program_version = "sslscan version 1.8.2\nhttp://www.titania.co.uk\nCopyright (C) Ian Ventura-Whiting 2009\n";
+const char *xml_version = "1.8.2";
 
 
 struct sslCipher
@@ -104,6 +105,8 @@ struct sslCheckOptions
 	int sslVersion;
 	int targets;
 	int pout;
+	int sslbugs;
+	int http;
 
 	// File Handles...
 	FILE *xmlOutput;
@@ -235,7 +238,7 @@ int tcpConnect(struct sslCheckOptions *options)
 {
 	// Variables...
 	int socketDescriptor;
-	unsigned char buffer[BUFFERSIZE];
+	char buffer[BUFFERSIZE];
 	struct sockaddr_in localAddress;
 	int status;
 
@@ -433,7 +436,15 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 	int socketDescriptor = 0;
 	SSL *ssl = NULL;
 	BIO *cipherConnectionBio;
+	BIO *stdoutBIO = NULL;
 	int tempInt;
+	char requestBuffer[200];
+	char buffer[50];
+	int resultSize = 0;
+
+	// Create request buffer...
+	memset(requestBuffer, 0, 200);
+	snprintf(requestBuffer, 199, "GET / HTTP/1.0\r\nUser-Agent: SSLScan\r\nHost: %s\r\n\r\n", options->host);
 
 	// Connect to host
 	socketDescriptor = tcpConnect(options);
@@ -455,10 +466,6 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 				// Connect SSL over socket
 				cipherStatus = SSL_connect(ssl);
 
-				// Disconnect SSL over socket
-				if (cipherStatus == 1)
-					SSL_shutdown(ssl);
-
 				// Show Cipher Status
 				if (!((options->noFailed == true) && (cipherStatus != 1)))
 				{
@@ -467,30 +474,95 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 					if (cipherStatus == 1)
 					{
 						if (options->xmlOutput != 0)
-							fprintf(options->xmlOutput, "accepted\" sslversion=\"");
+							fprintf(options->xmlOutput, "accepted\"");
 						if (options->pout == true)
 							printf("|| Accepted || ");
 						else
 							printf("    Accepted  ");
+						if (options->http == true)
+						{
+
+							// Stdout BIO...
+							stdoutBIO = BIO_new(BIO_s_file());
+							BIO_set_fp(stdoutBIO, stdout, BIO_NOCLOSE);
+
+							// HTTP Get...
+							SSL_write(ssl, requestBuffer, sizeof(requestBuffer));
+							memset(buffer ,0 , 50);
+							resultSize = SSL_read(ssl, buffer, 49);
+							if (resultSize > 9)
+							{
+								int loop = 0;
+								for (loop = 9; (loop < 49) && (buffer[loop] != 0) && (buffer[loop] != '\r') && (buffer[loop] != '\n'); loop++)
+								{ }
+								buffer[loop] = 0;
+
+								// Output HTTP code...
+								if (options->pout == true)
+									printf("%s || ", buffer + 9);
+								else
+								{
+									printf("%s", buffer + 9);
+									loop = strlen(buffer + 9);
+									while (loop < 17)
+									{
+										loop++;
+										printf(" ");
+									}
+								}
+								if (options->xmlOutput != 0)
+									fprintf(options->xmlOutput, " http=\"%s\"", buffer + 9);
+							}
+							else
+							{
+								// Output HTTP code...
+								if (options->pout == true)
+									printf("|| || ");
+								else
+									printf("                 ");
+							}
+						}
 					}
 					else if (cipherStatus == 0)
 					{
 						if (options->xmlOutput != 0)
-							fprintf(options->xmlOutput, "rejected\" sslversion=\"");
-						if (options->pout == true)
-							printf("|| Rejected || ");
+							fprintf(options->xmlOutput, "rejected\"");
+						if (options->http == true)
+						{
+							if (options->pout == true)
+								printf("|| Rejected || N/A || ");
+							else
+								printf("    Rejected  N/A              ");
+						}
 						else
-							printf("    Rejected  ");
+						{
+							if (options->pout == true)
+								printf("|| Rejected || ");
+							else
+								printf("    Rejected  ");
+						}
 					}
 					else
 					{
 						if (options->xmlOutput != 0)
-							fprintf(options->xmlOutput, "failed\" sslversion=\"");
-						if (options->pout == true)
-							printf("|| Failed || ");
+							fprintf(options->xmlOutput, "failed\"");
+						if (options->http == true)
+						{
+							if (options->pout == true)
+								printf("|| Failed || N/A || ");
+							else
+								printf("    Failed    N/A              ");
+						}
 						else
-							printf("    Failed    ");
+						{
+							if (options->pout == true)
+								printf("|| Failed || ");
+							else
+								printf("    Failed    ");
+						}
 					}
+					if (options->xmlOutput != 0)
+						fprintf(options->xmlOutput, " sslversion=\"");
 					if (sslCipherPointer->sslMethod == SSLv2_client_method())
 					{
 						if (options->xmlOutput != 0)
@@ -541,6 +613,10 @@ int testCipher(struct sslCheckOptions *options, struct sslCipher *sslCipherPoint
 						printf("%s\n", sslCipherPointer->name);
 				}
 
+				// Disconnect SSL over socket
+				if (cipherStatus == 1)
+					SSL_shutdown(ssl);
+
 				// Free SSL object
 				SSL_free(ssl);
 			}
@@ -848,7 +924,7 @@ int getCertificate(struct sslCheckOptions *options)
 									if (options->xmlOutput != 0)
 									{
 										fprintf(options->xmlOutput, "   <not-valid-after>");
-										ASN1_TIME_print(fileBIO, X509_get_notBefore(x509Cert));
+										ASN1_TIME_print(fileBIO, X509_get_notAfter(x509Cert));
 										fprintf(options->xmlOutput, "</not-valid-after>\n");
 									}
 								}
@@ -954,7 +1030,7 @@ int getCertificate(struct sslCheckOptions *options)
 											{
 												fprintf(options->xmlOutput, "    <extension name=\"");
 												i2a_ASN1_OBJECT(fileBIO, asn1Object);
-												BIO_printf(fileBIO, "\"%s>", tempInt2 ? " critical" : "");
+												BIO_printf(fileBIO, "\"%s>", tempInt2 ? " level=\"critical\"" : "");
 											}
 
 											// Print Extension value...
@@ -1066,7 +1142,9 @@ int testHost(struct sslCheckOptions *options)
 	// Test supported ciphers...
 	printf("\n%sTesting SSL server %s on port %d%s\n\n", COL_GREEN, options->host, options->port, RESET);
 	printf("  %sSupported Server Cipher(s):%s\n", COL_BLUE, RESET);
-	if (options->pout == true)
+	if ((options->http == true) && (options->pout == true))
+		printf("|| Status || HTTP Code || Version || Bits || Cipher ||\n");
+	else if (options->pout == true)
 		printf("|| Status || Version || Bits || Cipher ||\n");
 	sslCipherPointer = options->ciphers;
 	while ((sslCipherPointer != 0) && (status == true))
@@ -1077,6 +1155,12 @@ int testHost(struct sslCheckOptions *options)
 		if (options->ctx != NULL)
 		{
 
+			// SSL implementation bugs/workaround
+			if (options->sslbugs)
+				SSL_CTX_set_options(options->ctx, SSL_OP_ALL | 0);
+			else
+				SSL_CTX_set_options(options->ctx, 0);
+
 			// Load Certs if required...
 			if ((options->clientCertsFile != 0) || (options->privateKeyFile != 0))
 				status = loadCerts(options);
@@ -1226,6 +1310,14 @@ int main(int argc, char *argv[])
 		else if (strcmp("--tls1", argv[argLoop]) == 0)
 			options.sslVersion = tls_v1;
 
+		// SSL Bugs...
+		else if (strcmp("--bugs", argv[argLoop]) == 0)
+			options.sslbugs = 1;
+
+		// SSL HTTP Get...
+		else if (strcmp("--http", argv[argLoop]) == 0)
+			options.http = 1;
+
 		// Host (maybe port too)...
 		else if (argLoop + 1 == argc)
 		{
@@ -1300,6 +1392,9 @@ int main(int argc, char *argv[])
 			printf("                       client certificates.\n");
 			printf("  %s--starttls%s           If a STARTTLS is required to kick an\n", COL_GREEN, RESET);
 			printf("                       SMTP service into action.\n");
+			printf("  %s--http%s               Test a HTTP connection.\n", COL_GREEN, RESET);
+			printf("  %s--bugs%s               Enable SSL implementation  bug work-\n", COL_GREEN, RESET);
+			printf("                       arounds.\n");
 			printf("  %s--xml=<file>%s         Output results to an XML file.\n", COL_GREEN, RESET);
 			printf("  %s--version%s            Display the program version.\n", COL_GREEN, RESET);
 			printf("  %s--help%s               Display the  help text  you are  now\n", COL_GREEN, RESET);
